@@ -114,14 +114,15 @@ static gint m_GeometryY = -1;
 static Gok_Predictor m_pCommandPredictor = NULL;
 
 /* Private functions */
-gint gok_main_open(gint argc, gchar *argv[]);
-void gok_main_close (void);
-void gok_main_read_rc (void);
-void gok_main_initialize_access_methods (void);
-void gok_main_initialize_wordcomplete (void);
-void gok_main_initialize_commandprediction (void);
-gboolean gok_main_display_scan_main(void);
-void gok_main_display_geometry_error (void);
+static void gok_main_read_rc (void);
+static void gok_main_initialize_access_methods (void);
+static void gok_main_initialize_wordcomplete (void);
+static void gok_main_initialize_commandprediction (void);
+static gboolean gok_main_display_scan_main(void);
+static void gok_main_display_geometry_error (void);
+static void gok_main_object_state_listener (Accessible* pAccessible);
+static gboolean gok_main_has_xkb_extension ();
+static gboolean gok_main_check_sticky_keys (GtkWidget *widget);
 
 /* poptOption storage */
 static char* accessmethodname = NULL;
@@ -293,7 +294,8 @@ gint main (gint argc, gchar *argv[])
 *
 * returns: a program exit code.
 **/
-gint gok_main_open(gint argc, gchar *argv[])
+gint
+gok_main_open(gint argc, gchar *argv[])
 {
 	GnomeProgram *gok_program;
 	/* initialize member data */
@@ -419,7 +421,8 @@ gint gok_main_open(gint argc, gchar *argv[])
 	gok_spy_open();
 	gok_spy_register_windowchangelistener ((void *)gok_main_window_change_listener);
 	gok_spy_register_mousebuttonlistener ((void *)gok_main_mousebutton_listener);	
-
+	gok_spy_register_objectstatelistener ((void *)gok_main_object_state_listener);
+	
 	/* initialize sound */
 	gok_sound_initialize();
 
@@ -491,7 +494,7 @@ gok_log ("check for currently active frame");
 gok_log ("finished check");
 	
 	/* connect signal handlers; don't trap SIGKILL */ 
-	signal(SIGSEGV, (void (*)(int))gok_sig_handler);
+	signal(SIGSEGV, (void (*)(int))gok_sig_handler); 
 	signal(SIGTERM, (void (*)(int))gok_sig_handler);
 	signal(SIGINT,  (void (*)(int))gok_sig_handler);
 
@@ -499,7 +502,8 @@ gok_log ("finished check");
 }
 
 /* private */
-gboolean gok_main_display_scan_main()
+static gboolean
+gok_main_display_scan_main()
 {
 	if (gok_main_display_scan ((GokKeyboard*)NULL, mainkeyboardname, KEYBOARD_TYPE_UNSPECIFIED, KEYBOARD_LAYOUT_NORMAL, KEYBOARD_SHAPE_BEST) == FALSE)
 	{
@@ -651,6 +655,65 @@ void gok_main_window_change_listener (Accessible* pAccessible)
 	gok_log_leave();	
 }
 
+/**
+* gok_main_object_state_listener
+* @pAccessible: Pointer to the foreground window's accessible interface.
+* 
+* This function is called each time the foreground window changes.
+*
+*/
+static void 
+gok_main_object_state_listener (Accessible* pAccessible)
+{
+	GokKeyboard* pKB;
+	GokKey* pK;
+	AccessibleStateSet* ass = NULL;
+
+	pKB = gok_main_get_current_keyboard();
+	if (pKB != NULL) {
+		pK = pKB->pKeyFirst;
+		while (pK != NULL) {
+			if (pK->pNodeAccessible != NULL) {
+				if (pK->pNodeAccessible->paccessible == pAccessible) {
+					/* note: this is very specific for now because we are doing
+					   this to make sure checkbox and radio menu item state
+					   is represented properly. TODO: generalize
+					*/
+					ass = Accessible_getStateSet(pAccessible);
+					if (AccessibleStateSet_contains( ass, SPI_STATE_CHECKED )) {
+						pK->ComponentState.active = TRUE;
+					}
+					else {
+						pK->ComponentState.active = FALSE;
+					}
+					AccessibleStateSet_unref(ass);
+					gok_key_update_toggle_state (pK);
+ 					break;
+				}
+			}
+			pK = pK->pKeyNext;
+		}
+	}
+	/* notes:
+	   we now check to see if the current keyboard requires updating, currently
+	   we simply call gok_main_ds, but if this is too slow we can create a 
+	   method that updates the specific key in question 
+	*/
+	
+/*	AccessibleNode* pAN;
+	if (pKB != NULL) {
+		pAN = pKB->pNodeAccessible;
+		while (pAN != NULL) {
+			if (pAN->paccessible == pAccessible) {
+				gok_main_ds (pKB);
+				break;
+			}
+			pAN = pAN->pnext;
+		}
+	}*/
+}	
+
+
 /**
 * gok_main_motion_listener
 * @n_axes: Number of axes on which motion may have occurred
@@ -1273,7 +1336,8 @@ GtkWidget* gok_main_create_window (gboolean is_dock)
 * Retreives the name of the directory containing gok.rc from
 * GConf and loads it.
 */
-void gok_main_read_rc ()
+static void
+gok_main_read_rc ()
 {
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
@@ -1440,7 +1504,8 @@ GokKeyboard* gok_main_read_keyboards_from_dir (const char *directory)
 * Then calls gok_scanner_initialize passing the directory it
 * got from GConf.
 */
-void gok_main_initialize_access_methods ()
+static void
+gok_main_initialize_access_methods ()
 {
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
@@ -1480,7 +1545,8 @@ void gok_main_initialize_access_methods ()
 * Retreives the name of the directory containing dictionary.txt from
 * GConf and calls gok_wordcomplete_open passing that directory.
 */
-void gok_main_initialize_wordcomplete ()
+static void
+gok_main_initialize_wordcomplete ()
 {
     GConfClient *gconf_client = NULL;
     GError *gconf_err = NULL;
@@ -1518,7 +1584,8 @@ void gok_main_initialize_wordcomplete ()
 * Retreives the name of the directory containing dictionary.txt from
 * GConf and calls gok_wordcomplete_open passing that directory.
 */
-void gok_main_initialize_commandprediction ()
+static void
+gok_main_initialize_commandprediction ()
 {
 	gok_log_enter();
     /* intialize command predictor */
@@ -1824,7 +1891,8 @@ void gok_main_store_window_center ()
 * Delete any keyboards that were created.
 * This must be called at the end of the program.
 */
-void gok_main_close()
+void
+gok_main_close()
 {
 	/* NOTE: the order in which things are cleaned up is very important */
 
@@ -1833,6 +1901,7 @@ void gok_main_close()
 
 	/* unhook listeners/callbacks */
 	gok_spy_deregister_windowchangelistener ((void *)gok_main_window_change_listener);
+	gok_spy_deregister_objectstatelistener ((void *)gok_main_object_state_listener);
 	/* the call to deregister_mousebuttonlistener is in on_window1_destroy */
 	
 	gok_spy_stop();
@@ -1915,7 +1984,8 @@ void gok_main_display_gconf_error ()
 * 
 * Displays a dialog informing user that the given geometry was not correct.
 */
-void gok_main_display_geometry_error ()
+static void
+gok_main_display_geometry_error ()
 {
 	GtkWidget* dialog;
         
@@ -2029,7 +2099,7 @@ gok_main_get_login (void)
 * 
 * returns: gboolean
 */
-gboolean
+static gboolean
 gok_main_has_xkb_extension ()
 {
 	/* TODO/revisit - is this a good check? */
@@ -2040,7 +2110,7 @@ gok_main_has_xkb_extension ()
 	return FALSE;
 }
 
-gboolean
+static gboolean
 gok_main_check_sticky_keys (GtkWidget *widget)
 {
 	int op_rtn, event_rtn, error_rtn;
