@@ -36,8 +36,9 @@ extern "C" {
 GtkWidget* gok_main_create_window (gboolean is_dock);
 void gok_main_set_cursor (GdkCursor *cursor);
 gint gok_main_open(gint argc, gchar *argv[]);
+void gok_main_raise_window (void);
 GokKeyboard *gok_main_keyboard_find_byname (const gchar* NameKeyboard);
-gboolean gok_main_display_scan (GokKeyboard* pKeyboard, char* NameKeyboard, KeyboardTypes keyboardType, KeyboardLayouts layoutType);
+gboolean gok_main_display_scan (GokKeyboard* pKeyboard, gchar* nameKeyboard, KeyboardTypes typeKeyboard, KeyboardLayouts layout, KeyboardShape shape);
 gboolean gok_main_display_scan_previous (void);
 gboolean gok_main_display_scan_previous_premade (void);
 gboolean gok_main_display_scan_reset (void);
